#######################################################
# GLOBAL risks of the TSPM example                    #
# 17NRM05 EMUE Project - Activity A1.2.3              #
# Author: Francesca Pennecchi (INRIM)                 #
# Revised by Stephen Ellison (LGC)                    #
# Date: 09/10/2020    		                      #
#######################################################

rm(list=ls())

# GLOBAL RISKS for the consumer (Rc) and the producer (Rp)
# according to JCGM 106:2012, eqs. (19-20), with normal likelihood and lognormal prior pdf
# with a single upper acceptance limit AV and permitted upper tolerance limit TU 

um <- 0.07              		# Relative measurement uncertainty
TU <- 0.2                		# Tolerance limit
Avalues <- seq(0.15,0.25,0.001)  	# Values of acceptance limit AV (varying in [0.15, 0.25])
NAvalues <- length(Avalues)		# Lenght of vector Avalues
i <- 0					# Inizialization of running index

# Joint pdf to be integrated in the consumer's risk calculation
jointpdfC <- function(x, AV = 0.2, meanlog = 0, sdlog = 1, ux = 1){
pnorm((AV-x)/(ux*x)) * dlnorm(x,meanlog,sdlog) }

# Joint pdf to be integrated in the producer's risk calculation
jointpdfP <- function(x, AV = 0.2, meanlog = 0, sdlog = 1, ux = 1){
(1-pnorm((AV-x)/(ux*x))) * dlnorm(x,meanlog,sdlog) }


#######################
# Quarry 1
# INPUT DATA
Q1 <- scan('Q1data.txt')		# NOTE: the local path has to be set by the user
logQ1 <- log(Q1)			# Log-transformed data
ymeanlogQ1 <- mean(logQ1) 	 	# Sample mean
ystdlogQ1 <- sd(logQ1)     	 	# Sample standard deviation
ymeanQ1 <- mean(Q1)  	 		# Sample mean of the lognormally distributed data
ystdQ1 <- sd(Q1)     	 		# Sample standard deviation of the lognormally distributed data

# NORMALITY TEST LOG(Q)
ks.test(logQ1,"pnorm",mean(logQ1),sd(logQ1))
# D = 0.066218, p-value = 0.2896

RcQ1 <- rep(0,NAvalues)			# Inizialization of vector of Rc values
RpQ1 <- rep(0,NAvalues)			# Inizialization of vector of Rp values
for(A in Avalues)
{
  i <- i+1
  RcQ1[i] <- integrate(jointpdfC, TU, Inf, AV = A, meanlog = ymeanlogQ1, sdlog = ystdlogQ1, ux = um)$value
  RpQ1[i] <- integrate(jointpdfP, 0, TU, AV = A, meanlog = ymeanlogQ1, sdlog = ystdlogQ1, ux = um)$value
}
    
Avalues.required <- c(TU, 0.158, 0.172, 0.175, 0.220)
approx(Avalues, RcQ1, Avalues.required)  # 5.776505e-03 7.466221e-06 1.502437e-04 2.564651e-04 1.796521e-02
approx(Avalues, RpQ1, Avalues.required)  # 0.0073778817 0.0866044514 0.0489694298 0.0424287145 0.0005524898

max(Avalues[RcQ1<=0.0001]) # A should be smaller than 0.169 to have RcQ1 smaller than 0.01 % 
min(Avalues[RpQ1<=0.0001]) # A should be larger  than 0.23 to have RpQ1 smaller than 0.01 % 


#######################
# Quarry 2
# INPUT DATA
Q2 <- scan('Q2data.txt')		# NOTE: the local path has to be set by the user
logQ2 <- log(Q2)
ymeanlogQ2 <- mean(logQ2) 	 	# Sample mean
ystdlogQ2 <- sd(logQ2)     	 	# Sample standard deviation
ymeanQ2 <- mean(Q2)  	 		# Sample mean for the lognormally distributed data
ystdQ2 <- sd(Q2)     	 		# Sample standard deviation for the lognormally distributed data

# NORMALITY TEST LOG(Q)
ks.test(logQ2,"pnorm",mean(logQ2),sd(logQ2))
# D = 0.076056, p-value = 0.2605

RcQ2 <- rep(0,NAvalues)			# Inizialization of vector of Rc values
RpQ2 <- rep(0,NAvalues)			# Inizialization of vector of Rp values
i <- 0					# Inizialization of running index
for(A in Avalues)
{
  i <- i+1
  RcQ2[i] <- integrate(jointpdfC, TU, Inf, AV = A, meanlog = ymeanlogQ2, sdlog = ystdlogQ2, ux = um)$value
  RpQ2[i] <- integrate(jointpdfP, 0, TU, AV = A, meanlog = ymeanlogQ2, sdlog = ystdlogQ2, ux = um)$value
}
       
Avalues.required <- c(TU, 0.158, 0.172, 0.175, 0.220)
approx(Avalues, RcQ2, Avalues.required)  # 1.042315e-02 1.396142e-05 2.788562e-04 4.750819e-04 3.116272e-02
approx(Avalues, RpQ2, Avalues.required)  # 0.015228381 0.191168743 0.106162648 0.091504278 0.001114331


#######################
# Quarry 3
# INPUT DATA
Q3 <- scan('Q3data.txt')		# NOTE: the local path has to be set by the user
logQ3 <- log(Q3)
ymeanlogQ3 <- mean(logQ3) 	 	# Sample mean
ystdlogQ3 <- sd(logQ3)     	 	# Sample standard deviation
ymeanQ3 <- mean(Q3)  	 		# Sample mean for the lognormally distributed data
ystdQ3 <- sd(Q3)     	 		# Sample standard deviation for the lognormally distributed data


# NORMALITY TEST LOG(Q)
ks.test(logQ3,"pnorm",mean(logQ3),sd(logQ3))
# D = 0.089338, p-value = 0.4019

RcQ3 <- rep(0,NAvalues)			# Inizialization of vector of Rc values
RpQ3 <- rep(0,NAvalues)			# Inizialization of vector of Rp values
i <- 0					# Inizialization of running index
for(A in Avalues)
{
  i <- i+1
  RcQ3[i] <- integrate(jointpdfC, TU, Inf, AV = A, meanlog = ymeanlogQ3, sdlog = ystdlogQ3, ux = um)$value
  RpQ3[i] <- integrate(jointpdfP, 0, TU, AV = A, meanlog = ymeanlogQ3, sdlog = ystdlogQ3, ux = um)$value
}
             
Avalues.required <- c(TU, 0.158, 0.172, 0.175, 0.220)
approx(Avalues, RcQ3, Avalues.required)  # 4.596781e-03 6.023580e-06 1.208558e-04 2.061442e-04 1.409031e-02
approx(Avalues, RpQ3, Avalues.required)  # 0.006232563 0.077764927 0.042873064 0.036964936 0.000460868



#####################################################################
# PLOTS

# For the plot of the three quarries:
par(mfrow=c(1,3),mar=c(5,6,2,2))
# Plot for Q1~LogN(ymeanlogQ1, ystdlogQ1)
hist(Q1, prob=TRUE, xlim=c(0,0.3), ylim=c(0,12), cex.main = 2, font.axis = 6, cex.axis = 1.5, main = "Quarry 1", breaks=7, xlab = "", ylab = "")
title(family = "serif", ylab = "Frequency" , xlab = " ", cex.lab = 2, font.main = 6, font.lab = 6)  
mtext(family = "serif", cex = 1.5, text = expression( paste(italic(c)[m]/group("(",list(mg~m^-"\u0033"),")")) ),side = 1,line = 3.7)
ymeanQ1 <- mean(Q1)  	 		# Sample mean for the Lognormally distributed data
ystdQ1 <- sd(Q1)     	 		# Sample standard deviation for the Lognormally distributed data
lines(seq(0,0.3,0.001), dlnorm(seq(0,0.3,0.001), ymeanlogQ1, ystdlogQ1), type='l', lwd=2) # Prior LogN
# Plot for Q2~LogN(ymeanlogQ2, ystdlogQ2)
hist(Q2, prob=TRUE, xlim=c(0,0.3), ylim=c(0,12), cex.main = 2, font.axis = 6, cex.axis = 1.5, main = "Quarry 2", breaks=9, xlab = "", ylab = "")
title(family = "serif", ylab = "Frequency" , xlab = " ", cex.lab = 2, font.main = 6, font.lab = 6)  
mtext(family = "serif", cex = 1.5, text = expression( paste(italic(c)[m]/group("(",list(mg~m^-"\u0033"),")")) ),side = 1,line = 3.7)
ymeanQ2 <- mean(Q2)  	 		# Sample mean for the Lognormally distributed data
ystdQ2 <- sd(Q2)     	 		# Sample standard deviation for the Lognormally distributed data
lines(seq(0,0.3,0.001), dlnorm(seq(0,0.3,0.001), ymeanlogQ2, ystdlogQ2), type='l', lwd=2) # Prior LogN
# Plot for Q3~LogN(ymeanlogQ3, ystdlogQ3)
hist(Q3, prob=TRUE, xlim=c(0,0.25), ylim=c(0,12), cex.main = 2, font.axis = 6, cex.axis = 1.5, main = "Quarry 3", breaks=9, xlab = "", ylab = "")
title(family = "serif", ylab = "Frequency" , xlab = " ", cex.lab = 2, font.main = 6, font.lab = 6)  
mtext(family = "serif", cex = 1.5, text = expression( paste(italic(c)[m]/group("(",list(mg~m^-"\u0033"),")")) ),side = 1,line = 3.7)
ymeanQ3 <- mean(Q3)  	 		# Sample mean for the Lognormally distributed data
ystdQ3 <- sd(Q3)     	 		# Sample standard deviation for the Lognormally distributed data
lines(seq(0,0.25,0.001), dlnorm(seq(0,0.25,0.001), ymeanlogQ3, ystdlogQ3), type='l', lwd=2) # Prior LogN


# Plot of the risk values (consumer's risks in red - producer's risks in blue)
par(mfrow=c(1,3))

plot(Avalues,RcQ1, xlim=c(0.15,0.25), ylim=c(0,0.3), col = "red", type ="l", lwd=2, font.axis = 6, axes = FALSE, cex.axis=1.5, main = "", ylab = "", xlab = "")  
axis(1, xaxp=c(0.15,0.25,10), font.axis = 6, cex.axis=1.5)
axis(2, yaxp=c(0,0.3,3), font.axis = 6, cex.axis=1.5)
mtext(family = "serif", cex = 1.5, text = expression( paste(Acceptance~limit/group("(",list(mg~m^-"\u0033"),")")) ),side = 1,line = 3.7)
mtext(family = "serif", cex = 1.5, text = expression("Risk value"),side = 2,line = 2.5)
title(family = "serif", main="Global risks for quarry 1", xlab = " ", cex.lab = 2, font.main = 6, font.lab = 6, cex.main = 3)  
lines(Avalues,RpQ1,lwd=2,col = "blue") 

plot(Avalues,RcQ2, xlim=c(0.15,0.25), ylim=c(0,0.3), col = "red", type ="l", lwd=2, font.axis = 6, axes = FALSE, cex.axis=1.5, main = "", ylab = "", xlab = "")  
axis(1, xaxp=c(0.15,0.25,10), font.axis = 6, cex.axis=1.5)
axis(2, yaxp=c(0,0.3,3), font.axis = 6, cex.axis=1.5)
mtext(family = "serif", cex = 1.5, text = expression( paste(Acceptance~limit/group("(",list(mg~m^-"\u0033"),")")) ),side = 1,line = 3.7)
mtext(family = "serif", cex = 1.5, text = expression("Risk value"),side = 2,line = 2.5)
title(family = "serif", main="Global risks for quarry 2", xlab = " ", cex.lab = 2, font.main = 6, font.lab = 6, cex.main = 3)  
lines(Avalues,RpQ2,lwd=2,col = "blue")  

plot(Avalues,RcQ3, xlim=c(0.15,0.25), ylim=c(0,0.3), col = "red", type ="l", lwd=2, font.axis = 6, axes = FALSE, cex.axis=1.5, main = "", ylab = "", xlab = "")  
axis(1, xaxp=c(0.15,0.25,10), font.axis = 6, cex.axis=1.5)
axis(2, yaxp=c(0,0.3,3), font.axis = 6, cex.axis=1.5)
mtext(family = "serif", cex = 1.5, text = expression( paste(Acceptance~limit/group("(",list(mg~m^-"\u0033"),")")) ),side = 1,line = 3.7)
mtext(family = "serif", cex = 1.5, text = expression("Risk value"),side = 2,line = 2.5)
title(family = "serif", main="Global risks for quarry 3", xlab = " ", cex.lab = 2, font.main = 6, font.lab = 6, cex.main = 3)  
lines(Avalues,RpQ3,lwd=2,col = "blue")



